import numpy as np

# 线性回归（正规方程法）
class LinearRegression_Equation:
    # 根据公式计算w，是n*1的2维矩阵
    def fit(self, X, y):
        self.w = np.linalg.inv(X.T.dot(X)).dot(X.T).dot(y)
    
    # 根据公式(4.1)计算预测结果，是m*1的2维矩阵
    def predict(self, X):
        return X.dot(self.w)

# 线性回归（梯度下降法）
class LinearRegression_GD:
    def __init__(self,
        epochs = 1000,           # 训练轮数
        lr = 0.01           # 学习率
        ) -> None:
        self.epochs = epochs
        self.lr = lr
    
    def fit(self, X, y):
        m, n = X.shape
        w = np.zeros((n, 1))
        y = y.reshape(-1, 1)
        for _ in range(self.epochs):
            gradient = 1 / m * X.T.dot(X.dot(w) - y)
            w -= self.lr * gradient
        self.w = w
    
    def predict(self, X):
        return X.dot(self.w)